package drills.drill07.exercise3.replier

import javax.jms.Message
import javax.jms.Queue
import javax.jms.QueueConnectionFactory
import javax.jms.Session
import javax.jms.ObjectMessage
import javax.jms.TextMessage

abstract class AbstarctReplier(val queueName: String) : Replier {
    private val factory: QueueConnectionFactory = this.getFactory()
    private val connection = factory.createQueueConnection()
    private val session = connection.createQueueSession(
        false,
        Session.AUTO_ACKNOWLEDGE
    )
    private val queue = session.createQueue(queueName)
    private val receiver = session.createReceiver(queue)

    override fun onRequest(buildResponse: (Message) -> Message) {
        receiver.setMessageListener {
            val replyQueue = it.jmsReplyTo as Queue
            val sender = session.createSender(replyQueue)
            sender.send(buildResponse(it))
        }
    }

    override fun createTextMessage(text: String): TextMessage = session.createTextMessage(text)

    override fun createObjectMessage(): ObjectMessage = session.createObjectMessage()

    override fun createMessage(): Message = session.createMessage()

    override fun start() {
        connection.start()
    }

    override fun close() {
        session.close()
        connection.close()
    }

    abstract fun getFactory(uri: String = "tcp://localhost:61616"): QueueConnectionFactory
}
